package ParkNPark.middletier;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;

import ParkNPark.interfaces.ClientManager;
import ParkNPark.interfaces.ClientManagerFactoryPOA;
import ParkNPark.interfaces.ClientManagerHelper;
import ParkNPark.interfaces.InvalidClientException;
import ParkNPark.interfaces.ServiceUnavailableException;

/**
 * Creates and manages ClientManager instances for clients
 */
public class ClientManagerFactoryImpl extends ClientManagerFactoryPOA
{
	/**
     * Creates a new client manager instance for a new client
     * @throws ServiceUnavailableException Thrown if the database cannot be
     * contacted or if some other reason prevents the client manager from
     * being created successfully
     * @return A new client manager instance for a new client
     */
	public ClientManager getClientManager() throws ServiceUnavailableException
	{
		ClientManagerImpl impl = new ClientManagerImpl();
		try {
  		    return ClientManagerHelper.narrow(_poa().servant_to_reference(impl));
		} catch (Exception e) {
			System.err.println("Exception while trying to narrow a new ClientManagerImpl into a CORBA object: " + e.getClass().getName());
		    throw new ServiceUnavailableException("Could not attach new ClientManagerHelper to CORBA POA");
		}
	}

	/**
     * Returns the existing client manager instance for an existing client. A client can call
     * this method with its client ID and last sequence number on any server and will get its
     * client manager instance
     * @param clientID The ID of the client to get the ClientManager of
     * @throws InvalidClientException Thrown when the given client ID is not known to the system
     * @throws ServiceUnavailableException Thrown if the database cannot be contacted or if
     * some other reason prevents the client manager from being retrieved successfully
     * @return The existing client manager instance for an existing client
     */
	public ClientManager getExistingClientManager(int clientID)
	    throws ServiceUnavailableException, InvalidClientException
	{
		ClientManagerImpl impl = new ClientManagerImpl(clientID);
		try {
  		    return ClientManagerHelper.narrow(_poa().servant_to_reference(impl));
		} catch (Exception e) {
			System.err.println("Exception while trying to narrow a new ClientManagerImpl into a CORBA object: " + e.getClass().getName());
		    throw new ServiceUnavailableException("Could not attach new ClientManagerHelper to CORBA POA");
		}
	}
	
	/** Maximum number of attempts at the database */
	private int MAX_ATTEMPTS = 3;
	
	public void poke() throws ServiceUnavailableException {
		
		int numTries = 0;
		while (numTries < MAX_ATTEMPTS)
		{	try
			{	// Set database configuration
				String jdbcDriver = "com.mysql.jdbc.Driver";
				String jdbcURL = "jdbc:mysql://172.19.132.120:3306/ece749_team6";
				String jdbcUsername = "team6";
				String jdbcPassword = "procrastinators";
				
				// Load the JDBC driver
				try 
				{	Class.forName(jdbcDriver);
				} 
				catch (ClassNotFoundException e) 
				{	System.err.println("Could not load JDBC driver; ensure that it is within the classpath");
					throw new RuntimeException(e);
				}
				
				// Connect to the database
				Connection conn = DriverManager.getConnection(jdbcURL, jdbcUsername, jdbcPassword);
				conn.close();
				// TODO: Make connection persistent for performance
				
				return;
			} 
			catch (SQLException e) 
			{	numTries++;
			}
		}
		
		// If max attempts exceeded, throw exception
		
		System.err.println("Failed to execute commands on database, even after " + MAX_ATTEMPTS + " attempts.");
		throw new ServiceUnavailableException("Could not create connection.");
	
	}
}
